#pragma once

#include <set>

#include "Chart.h"
#include "elements/Axis.h"
#include "elements/Key.h"

////////////////////////////////////////////////////////////////////////////////////////////////////
//
// XYPlot
//
////////////////////////////////////////////////////////////////////////////////////////////////////

class QDESIGNER_WIDGET_EXPORT XYPlot : public Chart
{
	Q_OBJECT

	Q_ENUMS(SymbolType)

	public:
		enum SymbolType
		{
			Circle,
			Square,
			Diamond,
			Plus,
			CircleOutline,
			SquareOutline,
			DiamondOutline,
			Cross
		};

		XYPlot(QWidget *parent = 0);

		bool showKey() const;
		void setShowKey(bool showKey);

		double keyScale() const;
		void setKeyScale(double keyScale);

		double xMin() const;
		void setXMin(double xMin);

		double xMax() const;
		void setXMax(double xMax);

		int xPrecision() const;
		void setXPrecision(int xPrecision);

		double yMin() const;
		void setYMin(double yMin);

		double yMax() const;
		void setYMax(double yMax);

		int yPrecision() const;
		void setYPrecision(int yPrecision);

		const QStringList &labels() const;
		void setLabels(const QStringList &labels);

		int symbolSize() const;
		void setSymbolSize(int symbolSize);

		SymbolType symbol0() const;
		void setSymbol0(SymbolType symbol0);

		SymbolType symbol1() const;
		void setSymbol1(SymbolType symbol1);

		SymbolType symbol2() const;
		void setSymbol2(SymbolType symbol2);

		SymbolType symbol3() const;
		void setSymbol3(SymbolType symbol3);

		SymbolType symbol4() const;
		void setSymbol4(SymbolType symbol4);

		SymbolType symbol5() const;
		void setSymbol5(SymbolType symbol5);

		SymbolType symbol6() const;
		void setSymbol6(SymbolType symbol6);

		SymbolType symbol7() const;
		void setSymbol7(SymbolType symbol7);

		void addData(int dataSeries, double x, double y);
		void addData(const QString &label, double x, double y);
		void clearData(int dataSeries);
		void clearData(const QString &label);
		void clearAllData();

	protected:
		virtual void resetValues();
		virtual void generateRandomValues() = 0;

		void paintEvent(QPaintEvent *);
		virtual void paint(QPainter &painter);

		QPointF positionForValue(const std::pair<double, double> &value) const;

		Axis m_xAxis;
		Axis m_yAxis;

		double m_xMin;
		double m_xMax;
		int m_xPrecision;

		double m_yMin;
		double m_yMax;
		int m_yPrecision;

		double m_visibleXMin;
		double m_visibleXMax;
		double m_visibleYMin;
		double m_visibleYMax;

		std::shared_ptr<QStringList> m_labels;

		QRect m_chartRect;

		QFont m_font;
		QFontMetrics m_fontMetrics;

		int m_symbolSize;
		std::shared_ptr<QList<int>> m_symbols;

		Key m_key;

		using Entry = std::pair<double, double>;
		using EntryMultiSet = std::multiset<Entry, bool (*)(const Entry &, const Entry &)>;

		std::vector<EntryMultiSet> m_values;
};
